import * as firebase from 'firebase';
export declare enum AuthActionType {
    ApplyActionCode = 0,
    CheckActionCode = 1,
    ConfirmPasswordReset = 2,
    CreateUserWithEmailAndPassword = 3,
    SendPasswordResetEmail = 4,
    SetPersistence = 5,
    SignInAndRetrieveDataWithCredential = 6,
    SignInAnonymously = 7,
    SignInWithCredential = 8,
    SignInWithCustomToken = 9,
    SignInWithEmailAndPassword = 10,
    SignInWithPhoneNumber = 11,
    SignInWithPopup = 12,
    SignInWithRedirect = 13,
    SignOut = 14,
    UseDeviceLanguage = 15,
    VerifyPasswordResetCode = 16,
}
export declare enum DatabaseActionType {
    GoOffline = 0,
    GoOnline = 1,
}
export declare enum ReferenceActionType {
    Push = 0,
    Remove = 1,
    Set = 2,
    SetPriority = 3,
    SetWithPriority = 4,
    Transaction = 5,
    Update = 6,
}
export declare enum UserActionType {
    Unlink = 0,
    UpdateEmail = 1,
    UpdatePassword = 2,
    UpdatePhoneNumber = 3,
    UpdateProfile = 4,
}
export declare type ActionType = AuthActionType | DatabaseActionType | ReferenceActionType | UserActionType;
export interface FirebaseAction {
    actionType: ActionType;
    as: (category: string) => FirebaseAction;
    [propName: string]: any;
}
export interface UserProfile {
    displayName: null | string;
    photoURL: null | string;
}
export declare type ActionHandler = ((action: FirebaseAction) => Promise<any>);
export declare type UpdateFn = ((value: any) => any);
export declare const firebaseActions: {
    auth: {
        applyActionCode: (code: string) => FirebaseAction;
        checkActionCode: (code: string) => FirebaseAction;
        confirmPasswordReset: (code: string, newPassword: string) => FirebaseAction;
        createUserWithEmailAndPassword: (email: string, password: string) => FirebaseAction;
        sendPasswordResetEmail: (email: string) => FirebaseAction;
        setPersistence: (persistence: string) => FirebaseAction;
        signInAndRetrieveDataWithCredential: (credential: firebase.auth.AuthCredential) => FirebaseAction;
        signInAnonymously: () => FirebaseAction;
        signInWithCredential: (credential: firebase.auth.AuthCredential) => FirebaseAction;
        signInWithCustomToken: (token: string) => FirebaseAction;
        signInWithEmailAndPassword: (email: string, password: string) => FirebaseAction;
        signInWithPhoneNumber: (phoneNumber: string, verifier: firebase.auth.ApplicationVerifier) => FirebaseAction;
        signInWithPopup: (provider: firebase.auth.AuthProvider) => FirebaseAction;
        signInWithRedirect: (provider: firebase.auth.AuthProvider) => FirebaseAction;
        signOut: () => FirebaseAction;
        useDeviceLanguage: () => FirebaseAction;
        user: (user: firebase.User) => {
            unlink: () => FirebaseAction;
            updateEmail: (email: string) => FirebaseAction;
            updatePassword: (password: string) => FirebaseAction;
            updatePhoneNumber: (phoneNumber: string) => FirebaseAction;
            updateProfile: (profile: UserProfile) => FirebaseAction;
        };
        verifyPasswordResetCode: (code: string) => FirebaseAction;
    };
    database: {
        goOffline: () => FirebaseAction;
        goOnline: () => FirebaseAction;
        ref: (refPath: string) => {
            push: (value: any) => FirebaseAction;
            remove: () => FirebaseAction;
            set: (value: any) => FirebaseAction;
            setPriority: (priority: string | number | null) => FirebaseAction;
            setWithPriority: (value: any, priority: string | number | null) => FirebaseAction;
            transaction: (updateFn: (value: any) => any) => FirebaseAction;
            update: (values: any) => FirebaseAction;
        };
    };
};
export declare function makeActionHandler(app: firebase.app.App): ActionHandler;
