"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var AuthActionType;
(function (AuthActionType) {
    AuthActionType[AuthActionType["ApplyActionCode"] = 0] = "ApplyActionCode";
    AuthActionType[AuthActionType["CheckActionCode"] = 1] = "CheckActionCode";
    AuthActionType[AuthActionType["ConfirmPasswordReset"] = 2] = "ConfirmPasswordReset";
    AuthActionType[AuthActionType["CreateUserWithEmailAndPassword"] = 3] = "CreateUserWithEmailAndPassword";
    AuthActionType[AuthActionType["SendPasswordResetEmail"] = 4] = "SendPasswordResetEmail";
    AuthActionType[AuthActionType["SetPersistence"] = 5] = "SetPersistence";
    AuthActionType[AuthActionType["SignInAndRetrieveDataWithCredential"] = 6] = "SignInAndRetrieveDataWithCredential";
    AuthActionType[AuthActionType["SignInAnonymously"] = 7] = "SignInAnonymously";
    AuthActionType[AuthActionType["SignInWithCredential"] = 8] = "SignInWithCredential";
    AuthActionType[AuthActionType["SignInWithCustomToken"] = 9] = "SignInWithCustomToken";
    AuthActionType[AuthActionType["SignInWithEmailAndPassword"] = 10] = "SignInWithEmailAndPassword";
    AuthActionType[AuthActionType["SignInWithPhoneNumber"] = 11] = "SignInWithPhoneNumber";
    AuthActionType[AuthActionType["SignInWithPopup"] = 12] = "SignInWithPopup";
    AuthActionType[AuthActionType["SignInWithRedirect"] = 13] = "SignInWithRedirect";
    AuthActionType[AuthActionType["SignOut"] = 14] = "SignOut";
    AuthActionType[AuthActionType["UseDeviceLanguage"] = 15] = "UseDeviceLanguage";
    AuthActionType[AuthActionType["VerifyPasswordResetCode"] = 16] = "VerifyPasswordResetCode";
})(AuthActionType = exports.AuthActionType || (exports.AuthActionType = {}));
var DatabaseActionType;
(function (DatabaseActionType) {
    DatabaseActionType[DatabaseActionType["GoOffline"] = 0] = "GoOffline";
    DatabaseActionType[DatabaseActionType["GoOnline"] = 1] = "GoOnline";
})(DatabaseActionType = exports.DatabaseActionType || (exports.DatabaseActionType = {}));
var ReferenceActionType;
(function (ReferenceActionType) {
    ReferenceActionType[ReferenceActionType["Push"] = 0] = "Push";
    ReferenceActionType[ReferenceActionType["Remove"] = 1] = "Remove";
    ReferenceActionType[ReferenceActionType["Set"] = 2] = "Set";
    ReferenceActionType[ReferenceActionType["SetPriority"] = 3] = "SetPriority";
    ReferenceActionType[ReferenceActionType["SetWithPriority"] = 4] = "SetWithPriority";
    ReferenceActionType[ReferenceActionType["Transaction"] = 5] = "Transaction";
    ReferenceActionType[ReferenceActionType["Update"] = 6] = "Update";
})(ReferenceActionType = exports.ReferenceActionType || (exports.ReferenceActionType = {}));
var UserActionType;
(function (UserActionType) {
    UserActionType[UserActionType["Unlink"] = 0] = "Unlink";
    UserActionType[UserActionType["UpdateEmail"] = 1] = "UpdateEmail";
    UserActionType[UserActionType["UpdatePassword"] = 2] = "UpdatePassword";
    UserActionType[UserActionType["UpdatePhoneNumber"] = 3] = "UpdatePhoneNumber";
    UserActionType[UserActionType["UpdateProfile"] = 4] = "UpdateProfile";
})(UserActionType = exports.UserActionType || (exports.UserActionType = {}));
function action(actionType, props) {
    if (props === void 0) { props = {}; }
    return Object.assign({
        actionType: actionType,
        as: function (category) {
            return action(actionType, Object.assign({ category: category }, props));
        }
    }, props);
}
exports.firebaseActions = {
    auth: {
        applyActionCode: function (code) {
            return action(AuthActionType.ApplyActionCode, { code: code });
        },
        checkActionCode: function (code) {
            return action(AuthActionType.CheckActionCode, { code: code });
        },
        confirmPasswordReset: function (code, newPassword) {
            return action(AuthActionType.ConfirmPasswordReset, { code: code, newPassword: newPassword });
        },
        createUserWithEmailAndPassword: function (email, password) {
            return action(AuthActionType.CreateUserWithEmailAndPassword, {
                email: email,
                password: password
            });
        },
        sendPasswordResetEmail: function (email) {
            return action(AuthActionType.SendPasswordResetEmail, { email: email });
        },
        setPersistence: function (persistence) {
            return action(AuthActionType.SetPersistence, { persistence: persistence });
        },
        signInAndRetrieveDataWithCredential: function (credential) {
            return action(AuthActionType.SignInAndRetrieveDataWithCredential, {
                credential: credential
            });
        },
        signInAnonymously: function () { return action(AuthActionType.SignInAnonymously); },
        signInWithCredential: function (credential) {
            return action(AuthActionType.SignInWithCredential, { credential: credential });
        },
        signInWithCustomToken: function (token) {
            return action(AuthActionType.SignInWithCustomToken, { token: token });
        },
        signInWithEmailAndPassword: function (email, password) {
            return action(AuthActionType.SignInWithEmailAndPassword, { email: email, password: password });
        },
        signInWithPhoneNumber: function (phoneNumber, verifier) {
            return action(AuthActionType.SignInWithPhoneNumber, { phoneNumber: phoneNumber, verifier: verifier });
        },
        signInWithPopup: function (provider) {
            return action(AuthActionType.SignInWithPopup, { provider: provider });
        },
        signInWithRedirect: function (provider) {
            return action(AuthActionType.SignInWithRedirect, { provider: provider });
        },
        signOut: function () { return action(AuthActionType.SignOut); },
        useDeviceLanguage: function () { return action(AuthActionType.UseDeviceLanguage); },
        user: function (user) { return ({
            unlink: function () { return action(UserActionType.Unlink, { user: user }); },
            updateEmail: function (email) {
                return action(UserActionType.UpdateEmail, { email: email, user: user });
            },
            updatePassword: function (password) {
                return action(UserActionType.UpdatePassword, { password: password, user: user });
            },
            updatePhoneNumber: function (phoneNumber) {
                return action(UserActionType.UpdatePhoneNumber, { phoneNumber: phoneNumber, user: user });
            },
            updateProfile: function (profile) {
                return action(UserActionType.UpdateProfile, { profile: profile, user: user });
            }
        }); },
        verifyPasswordResetCode: function (code) {
            return action(AuthActionType.VerifyPasswordResetCode, { code: code });
        }
    },
    database: {
        goOffline: function () { return action(DatabaseActionType.GoOffline); },
        goOnline: function () { return action(DatabaseActionType.GoOnline); },
        ref: function (refPath) { return ({
            push: function (value) {
                return action(ReferenceActionType.Push, { refPath: refPath, value: value });
            },
            remove: function () { return action(ReferenceActionType.Remove, { refPath: refPath }); },
            set: function (value) { return action(ReferenceActionType.Set, { refPath: refPath, value: value }); },
            setPriority: function (priority) {
                return action(ReferenceActionType.SetPriority, { priority: priority, refPath: refPath });
            },
            setWithPriority: function (value, priority) {
                return action(ReferenceActionType.SetWithPriority, {
                    priority: priority,
                    refPath: refPath,
                    value: value
                });
            },
            transaction: function (updateFn) {
                return action(ReferenceActionType.Transaction, { refPath: refPath, updateFn: updateFn });
            },
            update: function (values) {
                return action(ReferenceActionType.Update, { refPath: refPath, values: values });
            }
        }); }
    }
};
function makeActionHandler(app) {
    var auth = app.auth();
    var db = app.database();
    function handleAction(action) {
        switch (action.actionType) {
            case AuthActionType.ApplyActionCode:
                return auth.applyActionCode(action.code);
            case AuthActionType.CheckActionCode:
                return auth.checkActionCode(action.code);
            case AuthActionType.ConfirmPasswordReset:
                return auth.confirmPasswordReset(action.code, action.newPassword);
            case AuthActionType.CreateUserWithEmailAndPassword:
                return auth.createUserWithEmailAndPassword(action.email, action.password);
            case AuthActionType.SendPasswordResetEmail:
                return auth.sendPasswordResetEmail(action.email);
            case AuthActionType.SetPersistence:
                return auth.setPersistence(action.persistence);
            case AuthActionType.SignInAndRetrieveDataWithCredential:
                return auth.signInAndRetrieveDataWithCredential(action.credential);
            case AuthActionType.SignInAnonymously:
                return auth.signInAnonymously();
            case AuthActionType.SignInWithCredential:
                return auth.signInWithCredential(action.credential);
            case AuthActionType.SignInWithCustomToken:
                return auth.signInWithCustomToken(action.token);
            case AuthActionType.SignInWithEmailAndPassword:
                return auth.signInWithEmailAndPassword(action.email, action.password);
            case AuthActionType.SignInWithPhoneNumber:
                return auth.signInWithPhoneNumber(action.phoneNumber, action.verifier);
            case AuthActionType.SignInWithPopup:
                return auth.signInWithPopup(action.provider);
            case AuthActionType.SignInWithRedirect:
                return auth.signInWithRedirect(action.provider);
            case AuthActionType.SignOut:
                return auth.signOut();
            case AuthActionType.UseDeviceLanguage:
                return auth.useDeviceLanguage();
            case AuthActionType.VerifyPasswordResetCode:
                return auth.verifyPasswordResetCode(action.code);
            case DatabaseActionType.GoOffline:
                return db.goOffline();
            case DatabaseActionType.GoOnline:
                return db.goOnline();
            case ReferenceActionType.Push:
                return new Promise(function (resolve) {
                    db
                        .ref(action.refPath)
                        .push(action.value)
                        .then(function (value) { return resolve(value); });
                });
            case ReferenceActionType.Remove:
                return db.ref(action.refPath).remove();
            case ReferenceActionType.Set:
                return db.ref(action.refPath).set(action.value);
            case ReferenceActionType.SetPriority:
                return db.ref(action.refPath).setPriority(action.priority, function () { return null; });
            case ReferenceActionType.SetWithPriority:
                return db
                    .ref(action.refPath)
                    .setWithPriority(action.value, action.priority);
            case ReferenceActionType.Transaction:
                return db.ref(action.refPath).transaction(action.updateFn);
            case ReferenceActionType.Update:
                return db.ref(action.refPath).update(action.values);
            case UserActionType.Unlink:
                return action.user.unlink();
            case UserActionType.UpdateEmail:
                return action.user.updateEmail(action.email);
            case UserActionType.UpdatePhoneNumber:
                return action.user.updatePhoneNumber(action.phoneNumber);
            case UserActionType.UpdateProfile:
                return action.user.updateProfile(action.profile);
            default:
                return new Promise(function (_resolve, reject) {
                    reject(new Error('No matching action type'));
                });
        }
    }
    return handleAction;
}
exports.makeActionHandler = makeActionHandler;
//# sourceMappingURL=actions.js.map